# frozen_string_literal: true

require "rest-client"

require_relative "helpers/json_parser"
require_relative "helpers/logger_helper"
require_relative "helpers/cluster_client/parsed_response_helper"
require_relative "errors/client_error"

module StatsCloud
  # statscloud.io cluster-api service client.
  class ClusterClient
    include StatsCloud::JsonParser
    include StatsCloud::LoggerHelper

    # Constructs cluster-api service client instance.
    #
    # @param [+String+] env
    #   cluster api environment
    #
    # @param [+String+] host
    #   cluster api host.
    #
    # @api public
    def initialize(env, host)
      @environment = env
      @host = host || "https://cluster-api-v1.statscloud.statscloud.io"
    end

    # Schedules cluster instance to be retrieved.
    #
    # @param [+String+] token
    #   authorization token.
    # @param [+String+] app
    #   cluster name.
    #
    # @api public
    def get_cluster(token, app)
      url = "#{host}/users/current/clusters/#{app}/#{environment}"
      response = http_client.get(url, headers(token))
      get_parsed_response(response)
    end

    # Schedules cluster to be (re)deployed.
    #
    # @param [+String+] token
    #   authorization token.
    # @param [+String+] app
    #   cluster name.
    # @param [+String+] configuration
    #   cluster configuration
    #
    # @api public
    def deploy_cluster(token, app, configuration)
      url = "#{host}/users/current/clusters/#{app}/#{environment}"
      body = body_hash_parameters(configuration)
      response = http_client.put(url, body.to_json, headers(token))
      get_parsed_response(response)
    rescue RestClient::ExceptionWithResponse => error
      error_info = convert_to_pretty_json(error.http_body)
      logger($stdout).error error_info
      raise statscloud_client_error(error_info)
    end

    # Schedules cluster to be undeployed.
    # @param [+String+] token
    #   authorization token.
    # @param [+String+] app
    #   cluster name.
    #
    # @api public
    def undeploy_cluster(token, app)
      url = "#{host}/users/current/clusters/#{app}/#{environment}/undeploy"
      response = http_client.post(url, nil, headers(token))
      get_parsed_response(response)
    end

    # Register statscloud client connection.
    # @param [+String+] token
    #   authorization token
    # @param [+String+] app
    #   cluster name
    # @param [+Array+] tags
    #   statscloud client tags
    def register_statscloud_connection(token, app, tags)
      url = "#{host}/users/current/clusters/#{app}/#{environment}/register-statscloud-connection"
      body = { tags: tags }.to_json
      response = http_client.post(url, body, headers(token))
      get_parsed_response(response)
    end

    private

    attr_reader :host, :environment

    def headers(token)
      default_headers.merge(auth_headers(token))
    end

    def default_headers
      {
        content_type: :json,
        accept: :json
      }
    end

    def auth_headers(token)
      {
        "auth-token" => token
      }
    end

    def body_hash_parameters(configuration)
      {
        configuration: configuration.to_json
      }
    end

    def statscloud_client_error(message)
      StatsCloud::ClientError.new(message)
    end

    def get_parsed_response(response)
      StatsCloud::ParsedResponseHelper.new(response)
    end

    def http_client
      RestClient
    end
  end
end
