# frozen_string_literal: true

require "fileutils"

module StatsCloud
  # This helper works to help log errors and info about StatsCloud service work.
  module LoggerHelper
    # Log information about work to standard output
    def logger(output = default_output, files_to_keep = 10, size_to_age = 1_024_000)
      file_utils.mkdir_p "log"
      @logger ||= create_logger_instance(output, files_to_keep, size_to_age)
    end

    def console_logger
      @console_logger ||= create_logger_instance(STDOUT)
    end

    # Logs information with 10 minutes interval
    def log_info(info_message)
      logged = log_with_interval(info_type, last_info_log_at, info_message)
      update_last_info_at && log_stop_logging("information") if logged
      true
    end

    # Logs errors with 10 minutes interval
    def log_error(error_message)
      logged = log_with_interval(error_type, last_error_log_at, error_message)
      update_last_error_at && log_stop_logging("errors") if logged
      false
    end

    private

    attr_reader :last_error_log_at, :last_info_log_at

    def default_output
      File.join("log", "statscloud.log")
    end

    def log_with_interval(type, last_log_at, message)
      logger.add(type, message) if can_log?(last_log_at)
    end

    def can_log?(last_log_at)
      last_log_at.nil? || Time.now.to_i - last_log_at > 600
    end

    def log_stop_logging(message_type)
      logger.info "Logging #{message_type} from #{self.class} will be stopped for 10 minutes."
    end

    def update_last_info_at
      @last_info_log_at = Time.now.to_i
    end

    def update_last_error_at
      @last_error_log_at = Time.now.to_i
    end

    def create_logger_instance(output, files_to_keep = nil, size_to_age = nil)
      logger_service.new(output, files_to_keep, size_to_age).tap do |log|
        log.progname = "StatsCloud"
      end
    end

    def info_type
      Logger::INFO
    end

    def error_type
      Logger::ERROR
    end

    def logger_service
      Logger
    end

    def file_utils
      FileUtils
    end
  end
end
