# frozen_string_literal: true

require "yaml"

module StatsCloud
  # This helper works to gather and merge the configuration from files.
  module AssetsHelper
    private

    def asset_selector_matches(filename, asset_selector)
      includes = asset_selector["include"]
      excludes = asset_selector["exclude"]
      process_match(includes, filename) && !process_match(excludes, filename)
    end

    def collect_statscloud_assets(config, source_mappings, dir = ".")
      asset_selector = get_checked_asset_selector(config)
      process_files(dir, config, source_mappings, asset_selector)
    end

    def join_configs(config, extra_config, source_mappings, file_name)
      process_array_fields(config, extra_config, source_mappings, file_name)
      process_object_fields(config, extra_config, source_mappings, file_name)
      process_value_fields(config, extra_config)
    end

    def get_checked_asset_selector(config)
      asset_selector = config["assetSelector"] || {
        "include" => ['.*\.statscloud.yml'], "exclude" => []
      }
      asset_selector["include"] = put_to_array_if_needed(asset_selector["include"])
      asset_selector["exclude"] = put_to_array_if_needed(asset_selector["exclude"])
      asset_selector
    end

    def get_config_from_file(file)
      return ::YAML.load_file(file) if File.exist?(file)

      {}
    end

    def process_files(dir, config, source_mappings, asset_selector)
      entries = Dir.entries(dir)
      entries.reduce({}) do |_config, entry|
        file_name = File.join(dir, entry)
        if it_directory?(file_name, entry)
          collect_statscloud_assets(config, source_mappings, file_name)
        elsif it_config_file?(file_name, asset_selector)
          extra_config = get_config_from_file(file_name)
          join_configs(config, extra_config, source_mappings, file_name)
        end
      end
    end

    def process_match(list, filename)
      list&.reduce(false) do |match, pattern|
        match || Regexp.new(pattern).match(filename)
      end
    end

    def process_array_fields(config, extra_config, source_mappings, file_name)
      array_fields.each do |array_field|
        next unless it_not_empty_array?(extra_config, array_field)

        config[array_field] ||= []
        source_mappings[array_field.to_sym].push(source_map_index(config, extra_config, array_field, file_name))
        config[array_field] = config[array_field].concat(extra_config[array_field])
      end
    end

    def process_object_fields(config, extra_config, source_mappings, file_name)
      object_fields.each do |object_field|
        next unless extra_config[object_field]&.kind_of?(Hash)

        source_mappings[object_field.to_sym].push(
          fields: extra_config[object_field].keys,
          sourceFile: file_name
        )
        config[object_field] = (config[object_field] || {}).merge(extra_config[object_field])
      end
    end

    def process_value_fields(config, extra_config)
      value_fields.each do |field|
        config[field] = extra_config[field] if config[field].nil? && !extra_config[field].nil?
      end
    end

    def put_to_array_if_needed(object)
      object.is_a?(Array) ? object : [object].compact
    end

    def it_directory?(file_name, entry)
      File.directory?(file_name) && !not_required_directory.include?(entry)
    end

    def it_config_file?(file_name, asset_selector)
      File.stat(file_name).file? && asset_selector_matches(file_name, asset_selector)
    end

    def it_not_empty_array?(extra_config, field)
      extra_config[field]&.kind_of?(Array) && extra_config[field].length.positive?
    end

    def source_map_index(config, extra_config, array_field, file_name)
      start_index = config[array_field].length
      {
        startIndex: start_index,
        endIndex: start_index + extra_config[array_field].length - 1,
        sourceFile: file_name
      }
    end

    def not_required_directory
      %w[. ..]
    end

    def array_fields
      %w[dashboards]
    end

    def object_fields
      %w[metrics admins alerts]
    end

    def value_fields
      %w[token application environment endpoint assetSelector propagateErrors
         passwordProtect retention flushIntervalInSeconds diskSize plugins]
    end
  end
end
