# frozen_string_literal: true

require "socket"

require_relative "../../errors/client_error"

module StatsCloud
  # This helper configures StatsCloud::Client.
  module StatsCloudHelper
    private

    def initialize_values(environment = nil, client_tags = nil)
      @config = {}
      @env ||= environment
      @tags ||= client_tags
      @source_mappings = { metrics: [], admins: [], dashboards: [], alerts: [] }
    end

    def generate_configuration(base_config)
      if base_config
        join_configs(@config, base_config, @source_mappings, "")
      else
        base_config = get_config_from_file(@config_file)
        join_configs(@config, base_config, @source_mappings, @config_file)
      end
    end

    def clear_data
      @config.delete("propagateErrors")
      @config.delete("token")
    end

    def config_values
      @config["sourceMaps"] = source_mappings
      config_token
      validate_configuration
    end

    def validate_configuration
      @app = @config["application"]
      raise error(app_name_missing_error_message) if @app.nil? || @app.empty?
    end

    def config_environment(env)
      @env = env
    end

    def config_tags(tags)
      @tags = tags
    end

    def update_config_file(file)
      @config_file = rails_environment ? config_dir_file(file) : file
    end

    def config_token
      @token = @config["token"] || ENV["STATSCLOUD_AUTH_TOKEN"]
    end

    def env
      @env ||= @config["environment"] || rails_environment || "default"
    end

    def tags
      @tags ||= @config["tags"] || [os.gethostname]
    end

    def log_successful_cluster_deploy
      log_info successful_message
      console_logger.info successful_message
    end

    def error(message = error_message)
      stop
      statscloud_error(message)
    rescue StandardError
      statscloud_error(message)
    end

    def cluster_error_message
      return "Unable to get information about cluster." unless @cluster

      @cluster["status"]["error"]
    end

    def app_name_missing_error_message
      "Unable to launch StatsCloud. Please configure application_name in path/to/config/file.yml."
    end

    def error_message
      "statscloud.io #{@app} cluster have failed to deploy. #{cluster_error_message}"
    end

    def successful_message
      "statscloud.io support configured, dashboard URLs are \n#{(@cluster['grafanaDashboardsUrls'] || []).join("\n")}"
    end

    def config_dir_file(file)
      file_name = file[0] == "." ? file.sub(/^./, "") : file
      File.join("config", file_name)
    end

    def rails_environment
      ENV["RAILS_ENV"]
    end

    def statscloud_error(message)
      StatsCloud::ClientError.new(message)
    end

    def os
      Socket
    end
  end
end
