# frozen_string_literal: true

require "leon"

module StatsCloud
  # This helper configures StatsmeterClient.
  module StatsmeterHelper
    private

    def initialize_plugin_threads(plugins)
      @plugins = plugins
      @mutex = mutex_service
    end

    def set_config(url, token, tags)
      @url = url
      @token = token
      @tags = tags.map { |t| t.gsub(/[^a-z0-9_-]/, "_") }
    end

    def set_client_to_nil
      @client = nil
    end

    def set_pending_values
      @pending_plain_events = buffer.new
      @pending_plain_offset = 0
      @pending_binary_events = buffer.new
      @pending_binary_offset = 0
    end

    def set_socket_values
      @names_map = {}
      @event_name_size_in_bytes = 0
    end

    def process_single_event(event)
      name = get_event_name(event)
      measurement = get_event_measurement(event)&.to_f

      return unless @names_map && @names_map[name]

      binary_length = get_binary_length(@event_name_size_in_bytes)
      plain_length = get_plain_length(name, measurement)

      flush_events if flush_condition(binary_length, plain_length)
      record_binary_event(name, measurement, binary_length)
      record_plain_event(name, measurement, plain_length)
    end

    def record_binary_event(name, measurement, binary_length)
      @pending_binary_events.writeInt32LE(@names_map[name], @pending_binary_offset)
      @pending_binary_events = @pending_binary_events.slice(0, @pending_binary_offset + @event_name_size_in_bytes)
      measurement.zero? ? record_binary_zero : record_binary_measurement(measurement)
      @pending_binary_offset += binary_length
    end

    def record_binary_measurement(data)
      @pending_binary_events.writeFloatBE(data, (@pending_binary_offset + @event_name_size_in_bytes))
    end

    def record_plain_measurement(data, name)
      @pending_plain_events.writeFloatBE(data, @pending_plain_offset + name.length + 1)
    end

    def record_binary_zero
      @pending_binary_events.writeInt32BE(0, (@pending_binary_offset + @event_name_size_in_bytes))
    end

    def record_plain_zero(name)
      record_zero(@pending_plain_events, @pending_plain_offset + name.length + 1)
    end

    def record_zero(buffer, offset)
      buffer.writeInt32BE(0, offset)
    end

    def record_plain_event(name, measurement, plain_length)
      @pending_plain_events.write(name, @pending_plain_offset)
      if measurement
        @pending_plain_events.write(",", @pending_plain_offset + name.length)
        measurement.zero? ? record_plain_zero(name) : record_plain_measurement(measurement, name)
      end
      @pending_plain_events.write(";", @pending_plain_offset + plain_length - 1)
      @pending_plain_offset += plain_length
    end

    def mutex_service
      Mutex.new
    end

    def buffer
      ::LEON::StringBuffer
    end
  end
end
