# frozen_string_literal: true

require "crc32"
require "eventmachine"

require_relative "helpers/statsmeter_client/socketio_helper"
require_relative "helpers/statsmeter_client/statsmeter_helper"
require_relative "helpers/statsmeter_client/event_helper"
require_relative "helpers/logger_helper"
require_relative "helpers/plugins_helper"

module StatsCloud
  # Client for Statsmeter.
  class StatsmeterClient
    include StatsCloud::LoggerHelper
    include StatsCloud::SocketIOHelper
    include StatsCloud::StatsmeterHelper
    include StatsCloud::EventHelper
    include StatsCloud::PluginsHelper

    # Maximum size of pending binary events buffer.
    BINARY_BUFFER_SIZE = 1024

    # Maximum size of pending binary events buffer.
    PLAIN_BUFFER_SIZE = 1024 * 100

    # Statsmeter cluster url is used to connect to cluster.
    #
    # Type: *String*
    attr_reader :url

    # Statscloud client tags.
    #
    # Type: *String*
    attr_reader :tags

    # Socket connection with statscloud cluster which is used to record events.
    #
    # Type: *SocketIO::Client::Simple*
    attr_accessor :client

    # Metric names.
    #
    # Type: *Hash*
    attr_accessor :names_map

    # Binary size for metric names.
    #
    # Type: *Integer*
    attr_accessor :event_name_size_in_bytes

    # Initialize statsmeter client.
    #
    # @param [+String+] url
    #   statsmeter url.
    # @param [+String+] token
    #   authorization token.
    def initialize(url, token, plugins, tags = [])
      initialize_plugin_threads(plugins)
      set_config(url, token, tags)
      set_client_to_nil
      set_pending_values
      set_socket_values
    end

    # Connects to the server and starts periodic sending events.
    def connect(register_connection_job = nil)
      @register_connection_job ||= register_connection_job
      Thread.new do
        connect_client
        start_plugins
        flush_events_loop
        @register_connection_job&.start
      end
    end

    # Records a single event.
    #
    # @param [+String+] name
    #   name of the event to record.
    # @param [+Integer+] measurement
    #   optional measurement, depending on metrics type.
    #
    # Save event in pending events.
    def record_event(name, measurement = 0)
      record_events(name: name, measurement: measurement)
    end

    # Records several events at once.
    #
    # @param [+Array+] events
    #   events to send (each should have name and optional measurement fields).
    #
    # Save events in pending events.
    def record_events(*events)
      record_events_array(events)
    end

    # Records an array of events at once.
    #
    # @param [+Array+] events
    #   array of events to send (each shoud have name and optional measurement fields).
    #
    # Save events in pending binary and plain arrays. Check flush condition.
    def record_events_array(events)
      events.each do |event|
        process_single_event(event)
      end
    rescue StandardError => error
      log_error error
    end

    # Sends all pending events to statscloud.
    def flush_events
      return if @pending_binary_offset.zero?

      checksum = crc32.calculate(@pending_plain_events.buffer, @pending_plain_offset, 0)
      return if checksum.zero?

      @pending_binary_events.writeInt32BE(checksum, @pending_binary_offset)
      send_message @pending_binary_events
      set_pending_values
    end

    # Shows statsmeter state.
    def connected?
      return false unless @client

      @client.state == :connect
    end

    # Stops socket.io connection.
    def close
      stop_eventmachine
      client&.auto_reconnection = false
      client&.disconnect
      set_client_to_nil
    end

    private

    def flush_events_loop
      Thread.new do
        eventmachine.run do
          eventmachine.add_periodic_timer(0.5) do
            @mutex.synchronize do
              connected? ? flush_events : connect.join
            end
          end
        end
      end
    end

    def start_plugins
      start_plugins_job(@plugins, @mutex)
    end

    def send_message(string_buffer)
      @client.emit :metric, binary_packet(string_buffer.buffer.bytes)
    end

    def binary_packet(byte_array)
      socketio_client.as_byte_buffer(byte_array)
    end

    def flush_condition(binary, plain)
      @pending_binary_offset + binary > BINARY_BUFFER_SIZE || @pending_plain_offset + plain > PLAIN_BUFFER_SIZE
    end

    def stop_eventmachine
      eventmachine.stop_event_loop if eventmachine.reactor_running?
    end

    def crc32
      Crc32
    end
  end
end
